﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.IO;

namespace howto_mask_saturated_pixels
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private enum PixelTypes
        {
            Red,
            Green,
            Blue,
        }

        private Bitmap OriginalImage = null;
        private Bitmap CurrentImage = null;

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdFile.ShowDialog() == DialogResult.OK)
            {
                OriginalImage = LoadBitmapUnlocked(ofdFile.FileName);
                CurrentImage = (Bitmap)OriginalImage.Clone();
                picResult.Image = CurrentImage;
                picResult.Visible = true;
                mnuFileSaveAs.Enabled = true;
                btnApply.Enabled = true;
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdFile.ShowDialog() == DialogResult.OK)
            {
                SaveImage(picResult.Image, sfdFile.FileName);
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            PixelTypes pixel_type = PixelTypes.Red;
            if (radGreen.Checked) pixel_type = PixelTypes.Green;
            else if (radBlue.Checked) pixel_type = PixelTypes.Blue;

            int diff = int.Parse(txtRedDiff.Text);
            CurrentImage = MaskPixels(CurrentImage, pixel_type, diff);
            picResult.Image = CurrentImage;
        }

        private Bitmap MaskPixels(Bitmap bitmap,
            PixelTypes pixel_type, int threshold)
        {
            Bitmap bm = new Bitmap(bitmap);
            Bitmap32 bm32 = new Bitmap32(bm);
            bm32.LockBitmap();

            int width = bm.Width;
            int height = bm.Height;
            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);

                    bool mask_pixel = false;
                    switch (pixel_type)
                    {
                        case PixelTypes.Red:
                            mask_pixel = ((r >= g + threshold) && (r > b + threshold));
                            break;
                        case PixelTypes.Green:
                            mask_pixel = ((g >= r + threshold) && (g > b + threshold));
                            break;
                        case PixelTypes.Blue:
                            mask_pixel = ((b >= r + threshold) && (b > g + threshold));
                            break;
                    }

                    if (mask_pixel)
                        bm32.SetPixel(x, y, 0, 0, 0, 255);
                    else
                        bm32.SetPixel(x, y, 255, 255, 255, 255);
                }
            }

            bm32.UnlockBitmap();
            return bm;
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        private void mnuFileReset_Click(object sender, EventArgs e)
        {
            CurrentImage = OriginalImage;
            picResult.Image = CurrentImage;
        }
    }
}